/*
 * Produced by:
 *	Peter Vale
 *	Side Effects Software Inc
 *	477 Richmond Street West
 *	Toronto, Ontario
 *	Canada   M5V 3E7
 *	416-504-9876
 *
 * NAME:	CPD_Track.C (Clip Public Domain Library, C++)
 *
 * COMMENTS:
 *		A set of raw data
 */


#include <iostream.h>
#include <string.h>

#include "CPD_Track.h"

#define	TRACK_HOLD	"hold"
#define	TRACK_SLOPE	"slope"
#define	TRACK_CYCLE	"cycle"
#define	TRACK_MIRROR	"mirror"
#define	TRACK_DEFAULT	"default"

//////////////////////////////////////////////////////////////////////////

CPD_Track::CPD_Track(void)
{
    data 	= 0;

    reset();
}

CPD_Track::~CPD_Track()
{
    reset();
}

void
CPD_Track::reset(void)
{
    name[0] = 0;
    delete [] data;
    data = 0;
    left_type		= TRACK_HOLD;
    right_type  	= TRACK_HOLD;
    default_value	= 0.0f;
}


////////////////////////////////////////////////////////////////////////////
//
// I/O methods
//
////////////////////////////////////////////////////////////////////////////

// Usually use cout as the ostream parameter

int
CPD_Track::save(ostream &os, int track_length) const
{
    int		i;

    os << "   {" << endl;

    os << "      name = " << name << endl;

    if (strcmp(left_type, TRACK_HOLD) != 0)
	os << "      lefttype = " << left_type << endl;

    if (strcmp(right_type, TRACK_HOLD) != 0)
	os << "      righttype = " << right_type << endl;

    if (default_value != 0.0F) 
	os << "      default = " << default_value << endl;


    if (track_length)
    {
	os << "      data = ";

	for (i=0; i<track_length; i++)
	    os << data[i] << " ";

	os << endl;
    }

    os << "   }" << endl;

    return 0;
}


int
CPD_Track::load(istream &is, int track_length)
{
    int		 i;
    char	 tmp_str[512]; 
    char	 equal_str[512];

    reset();

    is >> tmp_str;
    if (strcmp(tmp_str, "{") != 0)
    {
	cerr << "\rMissing Token: {" << endl;
	return 0;
    }

    do 
    {
	is >> tmp_str; 
	if (!strcmp(tmp_str, "}")) 
	    break;   // Successful end

	is >> equal_str;
	if (strcmp(equal_str, "=") != 0)
	{
	    cerr << "\rMissing Token: =" << endl;
	    return 0;
	}

	if (!strcmp(tmp_str, "lefttype"))
	{
	    is >> tmp_str;
	    switch ((tmp_str)[0])
	    {
		case 'd': left_type = TRACK_DEFAULT;break;
		case 'h': left_type = TRACK_HOLD;   break;
		case 'c': left_type = TRACK_CYCLE;  break;
		case 'm': left_type = TRACK_MIRROR; break;
		case 's': left_type = TRACK_SLOPE;  break;
		default:
		      cerr << "\rBad Token: " << tmp_str << endl;
	    }
	}
	else if (!strcmp(tmp_str, "righttype"))
	{
	    is >> tmp_str;
	    switch ((tmp_str)[0])
	    {
		case 'd': right_type = TRACK_DEFAULT;break;
		case 'h': right_type = TRACK_HOLD;   break;
		case 'c': right_type = TRACK_CYCLE;  break;
		case 'm': right_type = TRACK_MIRROR; break;
		case 's': right_type = TRACK_SLOPE;  break;
		default:
		      cerr << "\rBad Token: " << tmp_str << endl;
	    }
	}
	else if (!strcmp(tmp_str, "name"))	is >> name; 
	else if (!strcmp(tmp_str, "default"))   is >> default_value;
	else if (!strcmp(tmp_str, "data"))
	{
	    if (data)
		delete [] data;
	    data = new float[track_length];

	    for (i=0; i<track_length; i++)
		is >> data[i];
	}
	else cerr << "\rBad Token: " << tmp_str << endl;

    } while (!is.fail());

    return 0;
}

