#!/bin/csh -f

set PRMAN_VERSION = 3.9

set source = $1

#
#  Check to see if we're editing a .stree file
#  If we are, then start up shade-tree instead of openning the editor
#
set ext = $source:e
if ( "$ext" == "stree" && $?STREE_DIR ) then
    $STREE_DIR/bin/st -prman$PRMAN_VERSION -f $source
    exit 0
endif

xwsh -geometry 80x24+100+100 -title "Edit Shader $source" -e $EDITOR $source

set sl_path = $source:h
set sl_file = $source:t

if ( $sl_path != $sl_file ) then
    cd $sl_path
endif

if ( "$ext" == "sl" ) then
    shader $sl_file
else
    vcc -u $sl_file
endif

set base = $sl_file:r

#
#  Here, we install the dialog script in the users home directory
#  To change this, simply change the $HOME/houdini to be a different
#  location in the HOUDINI_PATH variable (i.e.
#  /usr/local/houdini/config/Scripts)
#
set ds_path = $HOME/houdini/config/Scripts

# Make sure that the directory exists
if ( ! -d $ds_path ) then
    mkdir -p $ds_path
    if ( ! -d $ds_path ) then
	/usr/sbin/confirm -t "Error creating dialog script:" \
			  -t "the target directory does not exist."
	exit 1
    endif
endif

# Now, make a dialog script for the slo file, put in /tmp to prevent
# overwriting of other RMshader.ds dialogs

if ( "$ext" == "sl") then
    # Clear out previous dialog script
    foreach dir ( surface light displace fog interior )
	set ext = /tmp/$dir/$base.ds
	if ( -r $ext ) then
	    rm -f $ext
	endif
    end

    rmands -d /tmp $base.slo

    set ext = unknown

    set dir = /tmp/surface/$base.ds
    if ( -r $dir ) then
	set ext = shade
	set script = RMshader.ds
    else
	set dir = /tmp/light/$base.ds
	if ( -r $dir ) then
	    set ext = light
	    set script = RMlight.ds
	else
	    set dir = /tmp/displace/$base.ds
	    if ( -r $dir ) then
		set ext = disp
		set script = RMdisplace.ds
	    else
		set dir = /tmp/interior/$base.ds
		if ( -r $dir ) then
		    set ext = fog
		    set script = RMinterior.ds
		endif
	    endif
	endif
    endif

    set ds_file = $base.$ext.ds
    mv $dir $ds_path/$ds_file
    set doit = 1
    if ( -r $ds_path/$script ) then
	set count = `egrep "$ds_file" $ds_path/$script | wc -l`
	if ( $count != 0 ) then
	    set doit = 0
	endif
    endif

    if ( $doit == "1" ) then
	echo "include $ds_file" >> $ds_path/$script
    endif

    if ( $ext == unknown ) then
	/usr/sbin/confirm -t "No shaders found for dialog scripts"
	exit 1
    endif
else
    # Here, we assume that it's VEX
    set ext = unknown
    if ( -r $base.ds ) then
	set context = `head -1 $base.ds | cut -d " " -f 3`

	switch ( $context )
            case Surface
		set ext = vsurf;
		set script = VEXsurface.ds
		breaksw;
            case Displacement
		set ext = vdisp;
		set script = VEXdisplace.ds
		breaksw;
            case Light
		set ext = vlight;
		set script = VEXlight.ds
		breaksw;
            case Shadow
		set ext = vshadow;
		set script = VEXshadow.ds
		breaksw;
            case Fog
		set ext = vfog;
		set script = VEXfog.ds
		breaksw;
	endsw

	set ds_file = $base.$ext.ds
	mv $base.ds $ds_path/$ds_file

	# Now, we use the include mechanism of the dialog scripts to include
	# the correct .ds file.  We want to avoid including the shader more
	# than once, so we check it in the if statement below.
	set doit = 1
	if ( -r $ds_path/$script ) then
	    set count = `egrep "$ds_file" $ds_path/$script | wc -l`
	    if ( $count != 0 ) then
		set doit = 0
	    endif
	endif

	if ( $doit == "1" ) then
	    echo "include $ds_file" >> $ds_path/$script
	endif
    endif

    if ( $ext == unknown ) then
	/usr/sbin/confirm -t "No shaders found for dialog scripts"
	exit 1
    endif
endif
exit 0
