/*
 * PROPRIETARY INFORMATION.  This software is proprietary to
 * Side Effects Software Inc., and is not to be reproduced,
 * transmitted, or disclosed in any way without written permission.
 *
 * Produced by:
 *      Side Effects
 *      477 Richmond Street West
 *      Toronto, Ontario
 *      Canada   M5V 3E7
 *      416-504-9876
 *
 * NAME:	IFD library (C++)
 *
 * COMMENTS:	Output an IFD to custom renderer
 *
 * Since the renderer does not support rendering of multiple frames
 * at once, we don't worry about the openMultiFrame() and closeMultiFrame()
 * methods.
 *
 */

#ifndef __IFD_CustomRender_h__
#define __IFD_CustomRender_h__

#include <UT/UT_DMatrix4.h>
#include <IFD/IFD_Output.h>

class IFD_CustomRender : public IFD_Output {
public:
	     IFD_CustomRender();
    virtual ~IFD_CustomRender();

    virtual const char	*renderer(void) const;
    virtual void	 getOutputFlags(IFD_OutputFlags &flags) const;
    virtual int		 doSingleFrame(IFD_Scene &, IFD_RenderParms &, float t);

private:
    void	outputMatrix(const char *token, UT_DMatrix4 &xform);

    void	writeHeader(IFD_Scene &scene);
    void	outputLights(IFD_Scene &scene);
    void	outputObjects(IFD_Scene &scene);
    void	writeTrailer(IFD_Scene &scene);

private:
    ostream	*os;				// Output stream
    float	 myStartTime, myEndTime;	// Start & End time
    int		 myObjectCount, myLightCount;
    int		 myBinaryFlag;
    UT_DMatrix4	 myCameraTransform;
};

#endif
