/*
 * PROPRIETARY INFORMATION.  This software is proprietary to
 * Side Effects Software Inc., and is not to be reproduced,
 * transmitted, or disclosed in any way without written permission.
 *
 * Produced by:
 *      Side Effects
 *      477 Richmond Street West
 *      Toronto, Ontario
 *      Canada   M5V 3E7
 *      416-504-9876
 *
 * NAME:	IFD library (C++)
 *
 * COMMENTS:	Output an IFD to custom renderer
 *
 * Since the renderer does not support rendering of multiple frames
 * at once, we don't worry about the openMultiFrame() and closeMultiFrame()
 * methods.
 *
 */

#ifndef __IFD_SimpleRIB_h__
#define __IFD_SimpleRIB_h__

#include <UT/UT_DMatrix4.h>
#include <IFD/IFD_Output.h>

class IFD_SimpleRib : public IFD_Output {
public:
	     IFD_SimpleRib();
    virtual ~IFD_SimpleRib();

    virtual const char	*renderer(void) const;
    virtual void	 getOutputFlags(IFD_OutputFlags &flags) const;
    virtual int		 doSingleFrame(IFD_Scene &, IFD_RenderParms &, float t);

private:
    void	writeHeader(IFD_Scene &scene);
    void	outputLights(IFD_Scene &scene);
    void	outputGeometry(IFD_Scene &scene);
    void	outputObjects(IFD_Scene &scene);
    void	writeTrailer();

private:
    void	 outputMatrix(const UT_DMatrix4 &xform, int change_space=1);
    void	 saveRibGeometry(const GU_Detail *gdp);

    void	**myLightHandles;
    void	**myGeometryHandles;
    float	  myStartTime, myEndTime;	// Start & End time
    int		  myObjectCount, myGeometryCount, myLightCount;
    int		  myBinaryFlag;
    UT_DMatrix4	  myCameraTransform;
};

#endif
