/*
 * PROPRIETARY INFORMATION.  This software is proprietary to
 * Side Effects Software Inc., and is not to be reproduced,
 * transmitted, or disclosed in any way without written permission.
 *
 * Produced by:
 *	Side Effects Software Inc
 *	477 Richmond Street West
 *	Toronto, Ontario
 *	Canada   M5V 3E7
 *	416-504-9876
 *
 * NAME:	ROP_SimpleRib.h (  ROP Library, C++)
 *
 * COMMENTS:	This code is a ROP which uses the IFD_SimpleRib code (../IFD)
 *		to build a custom ROP for the render class.
 */

#ifndef __ROP_SimpleRib__
#define __ROP_SimpleRib__

#include <ROP/ROP_IFD.h>

class OP_OperatorTable;

class ROP_SimpleRib : public ROP_IFD
{
public:
    virtual unsigned	 disableParms();
    virtual const char	*getRenderName();

    static CH_LocalVariable	myVariableList[];
    static PRM_Template         *getCommandTemplate();
    static OP_Node              *myConstructor(OP_Network *net,
					       const char*name,
					       OP_Operator *op);

    static void		installOperator(OP_OperatorTable *table);

protected:

    ROP_SimpleRib(OP_Network *net, const char *name, OP_Operator *entry);
    virtual ~ROP_SimpleRib();

    virtual IFD_Entry	*getRenderType();
    virtual void	 setRendererParms(IFD_Output *renderer);
    virtual void	 getFrameDevice(UT_String &str, float t,
					const char *rtype);
    
    void	 DEVICE(UT_String &str, float t)
		 {
		     // If we aren't doing a raster render (i.e. for the render
		     // COP), we should evaluate the device string, otherwise,
		     // the getRenderDevice() will get the correct device for
		     // output.
		     if (getRenderMode() == RENDER_PRM)
			  evalString(str, "device", &rmanIndirect[0], 0, t);
		     else str = getRenderDevice();
		 }
private:
    static int	*rmanIndirect;
};

#endif
