/*
 * PROPRIETARY INFORMATION.  This software is proprietary to
 * Side Effects Software Inc., and is not to be reproduced,
 * transmitted, or disclosed in any way without written permission.
 *
 * Produced by:
 *      Side Effects
 *      477 Richmond Street West
 *      Toronto, Ontario
 *      Canada   M5V 3E7
 *      416-504-9876
 *
 * NAME:	SOP Library (C++)
 *
 * COMMENTS:	The BlindData SOP.  This SOP simply saves some additional
 *		information to the HIP file.  It's cook method is simply copies
 *		the source geometry.
 */

// For saving and loading we need to know about CPIO packets...
#include <UT/UT_CPIO.h>

#include <UT/UT_DSOVersion.h>
#include <GU/GU_Detail.h>
#include <PRM/PRM_Include.h>
#include <OP/OP_Operator.h>
#include <OP/OP_OperatorTable.h>
#include "SOP_BlindData.h"

void
newSopOperator(OP_OperatorTable *table)
{
     table->addOperator(new OP_Operator("blinddata",
					"BlindData",
					 SOP_BlindData::myConstructor,
					 SOP_BlindData::myTemplateList,
					 1,
					 1,
					 0));
}

PRM_Template
SOP_BlindData::myTemplateList[] = {
    PRM_Template(),
};


OP_Node *
SOP_BlindData::myConstructor(OP_Network *net, const char *name, OP_Operator *op)
{
    return new SOP_BlindData(net, name, op);
}

SOP_BlindData::SOP_BlindData(OP_Network *net, const char *name, OP_Operator *op)
	: SOP_Node(net, name, op)
{
}

SOP_BlindData::~SOP_BlindData() {}

OP_ERROR
SOP_BlindData::cookMySop(OP_Context &context)
{
    // Before we do anything, we must lock our inputs.  Before returning,
    //	we have to make sure that the inputs get unlocked.
    if (lockInputs(context) >= UT_ERROR_ABORT)
	return error();

    duplicateSource(0, context);

    unlockInputs();
    return error();
}

const char *
SOP_BlindData::inputLabel(unsigned) const
{
    return "Source Geometry";
}

static const char	*theExtension = "mydata";

OP_ERROR
SOP_BlindData::save(ostream &os, const OP_SaveFlags &flags,
		    const char *path_prefix)
{
    UT_CPIO	 packet;
    char	 path[UT_SMALLBUF];

    clearErrors();

    // First, let the base class save its stuff.
    if (SOP_Node::save(os, flags, path_prefix) >= UT_ERROR_ABORT)
	return error();

    ostrstream ts(path, UT_SMALLBUF); ts << path_prefix << getName() << "."
				  << theExtension << ends;
    packet.open(os, path);
    savePrivateData(os, flags.isBinary());
    packet.close(os);

    return error();
}

OP_ERROR
SOP_BlindData::load(istream &is, int binary, const char *extension,
		    int filesize, const char *path)
{
    if (!strcmp(extension, theExtension))
    {
	// Here's my blind data!
	loadPrivateData(is, binary);
	return error();
    }
    return SOP_Node::load(is, binary, extension, filesize, path);
}

int
SOP_BlindData::loadPrivateData(istream &is, int binary)
{
    UT_String	data;

    data.load(is, binary);
    return !is ? 0 : 1;
}

int
SOP_BlindData::savePrivateData(ostream &os, int binary)
{
    UT_String	data;

    data = "This is my private data";
    data.save(os, binary);
    return !os ? 0 : 1;
}
