/*
 * PROPRIETARY INFORMATION.  This software is proprietary to
 * Side Effects Software Inc., and is not to be reproduced,
 * transmitted, or disclosed in any way without written permission.
 *
 * Produced by:
 *      Side Effects
 *      477 Richmond Street West
 *      Toronto, Ontario
 *      Canada   M5V 3E7
 *      416-504-9876
 *
 * NAME:	SOP library (C++)
 *
 * COMMENTS:	This SOP builds a star.
 */

#ifndef __SOP_Star_h__
#define __SOP_Star_h__

#include <SOP/SOP_Node.h>

class SOP_Star : public SOP_Node
{
public:
    static OP_Node		*myConstructor(OP_Network*, const char *,
							    OP_Operator *);
    static PRM_Template		 myTemplateList[];

    // You only have to have this data if you have local variables
    static CH_LocalVariable	 myVariables[];

protected:
	     SOP_Star(OP_Network *net, const char *name, OP_Operator *op);
    virtual ~SOP_Star();

    virtual unsigned		 disableParms();
    virtual OP_ERROR		 cookMySop(OP_Context &context);

    // You only have to write this function if you have local variables
    virtual float		 getVariableValue(int index, int thread);

private:
    int		DIVISIONS(float t)	{ return evalInt  (0, 0, t); }
    float	XRADIUS(float t)	{ return evalFloat(1, 0, t); }
    float	YRADIUS(float t)	{ return evalFloat(1, 1, t); }
    int		NEGRADIUS()		{ return evalInt  (2, 0, 0); }
    float	CENTERX(float t) 	{ return evalFloat(3, 0, t); }
    float	CENTERY(float t) 	{ return evalFloat(3, 1, t); }
    float	CENTERZ(float t) 	{ return evalFloat(3, 2, t); }
    int		ORIENT()		{ return evalInt  (4, 0, 0); }

    int		myCurrPoint;
    int		myTotalPoints;
};

#endif
